# Basic Python Plugin Example
#
# Author: Zack
#
"""
<plugin key="Controlsys-Ammeter" name="Controlsys Ammeter" author="Zack" version="1.0.0" externallink="http://domoticz.cn/forum/">
    <params>
        <param field="Address" label="IP" width="100px" required="true" default=""/>
        <param field="Mode1" label="检测间隔(秒)" width="40px" required="true" default="60"/>
    </params>
</plugin>
"""
import Domoticz
import socket
import json
import threading

class plugin:
	def UpdateDevice(self, Unit, nValue, sValue):
		if (Unit in Devices):
			if (Devices[Unit].nValue != nValue) or (Devices[Unit].sValue != sValue):
				Devices[Unit].Update(nValue=nValue, sValue=str(sValue))
				Domoticz.Log("Update "+str(nValue)+":'"+str(sValue)+"' ("+Devices[Unit].Name+")")

	def onStart(self):
		Domoticz.Heartbeat(int(Parameters["Mode1"]))
		Domoticz.Connection(Name="Data Connection", Transport="TCP/IP",Address=Parameters["Address"], Protocol="line", Port="8899").Connect()

		if ( 1 not in Devices):  
			Domoticz.Device(Name="电流", Unit=1, Type=243, Subtype=23, Used=1).Create()

		if ( 2 not in Devices):  
			Domoticz.Device(Name="电压", Unit=2, Type=243, Subtype=8,  Used=1).Create()

		if ( 3 not in Devices):
			 Domoticz.Device(Name="功率", Unit=3, TypeName="kWh", Used=1).Create()

	def onStop(self):
		Domoticz.Log("onStop called")
		if self.Conn.Connected():
			self.Conn.Disconnect()

	def onConnect(self, Connection, Status, Description):
		Domoticz.Log("onConnect called "+Connection.Address)
		self.Conn = Connection

	def onMessage(self, Connection, Data, Status, Extra):
		Domoticz.Log("onMessage called ")
		self.readable_info(Data)

	def readable_info(self, Data):
		info = self.show_info(Data)
		if info is None or len(info) == 0:
			return ""
		I = "%.2f" % (float(info["I"])/100.0)
		U = "%.2f" % (float(info["U"])/100.0)
		# F = "%.2f" % (float(info["F"])/100.0)
		P = "%.2f" % (float(info["P"])/10.0)
		E = info["E"]

		self.UpdateDevice(1,0,I)
		self.UpdateDevice(2,0,U)
		self.UpdateDevice(3,0,P+";"+E)
		
	

	def show_info(self, recv):
		recv = recv.decode('utf-8')
		if recv is None or len(recv) == 0:
			return None
		info = recv[4:-4].split(",")
		return  {
			"I": info[0] if len(info[0]) != 0 else "0",
			"U": info[1] if len(info[1]) != 0 else "0",
			# "F": info[2] if len(info[2]) != 0 else "0",
			"P": info[3] if len(info[3]) != 0 else "0",
			"E": info[5] if len(info[5]) != 0 else "0",
			}
		

	def onCommand(self, Unit, Command, Level, Hue):
		Domoticz.Log("onCommand called for Unit " + str(Unit) + ": Parameter '" + str(Command) + "', Level: " + str(Level))

	def onNotification(self, Name, Subject, Text, Status, Priority, Sound, ImageFile):
		Domoticz.Log("Notification: " + Name + "," + Subject + "," + Text + "," + Status + "," + str(Priority) + "," + Sound + "," + ImageFile)

	def onDisconnect(self, Connection):
		Domoticz.Log("onDisconnect called")
		Domoticz.Log("try connect again...")
		Domoticz.Connection(Name="Data Connection", Transport="TCP/IP",Address=Parameters["Address"], Protocol="line", Port="8899").Connect()

	def onHeartbeat(self):
		if self.Conn.Connected():
			self.Conn.Send("AT+YZOUT\r\n")

global _plugin
_plugin = plugin()

def onStart():
	global _plugin
	_plugin.onStart()

def onStop():
	global _plugin
	_plugin.onStop()

def onConnect(Connection, Status, Description):
	global _plugin
	_plugin.onConnect(Connection, Status, Description)

def onMessage(Connection, Data, Status, Extra):
	global _plugin
	_plugin.onMessage(Connection, Data, Status, Extra)

def onCommand(Unit, Command, Level, Hue):
	global _plugin
	_plugin.onCommand(Unit, Command, Level, Hue)

def onNotification(Name, Subject, Text, Status, Priority, Sound, ImageFile):
	global _plugin
	_plugin.onNotification(Name, Subject, Text, Status, Priority, Sound, ImageFile)

def onDisconnect(Connection):
	global _plugin
	_plugin.onDisconnect(Connection)

def onHeartbeat():
	global _plugin
	_plugin.onHeartbeat()